#ifndef LIBKYDATASEC_H
#define LIBKYDATASEC_H

#ifdef __cplusplus
extern "C" {
#endif

//扩展错误码errno
#define   EALGO         (500)       //算法错误


typedef enum _kdk_algorithm_type {
    ALGO_TYPE_SOFT,     // 软算法（openssl）

    // 暂不支持
    //ALGO_TYPE_UKEY,     // UKey算法（SKF等国标算法）
    //ALGO_TYPE_TPM,      // 可信芯片算法
    //ALGO_TYPE_TPCM,     // 可信芯片算法
    ALGO_TYPE_NONE
}kdk_algorithm_type;

/**
 * @brief 对称加密，使用AES算法（256位）CBC模式进行对称加密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param plaindata [in] 原数据
 * @param plaindatalen [in] 元数据长度
 * @param cipherdata [in/out] 密文数据
 * @param cipherdatalen [in/out] 密文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_encrypt_aes256_cbc(const unsigned char *key, unsigned int keylen,
                                    const unsigned char *iv, unsigned int ivlen,
                                    const unsigned char *plaindata, unsigned int plaindatalen,
                                    unsigned char *cipherdata, unsigned int *cipherdatalen);
/**
 * @brief 对称解密，使用AES算法（256位）CBC模式进行对称解密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param cipherdata [in] 密文数据
 * @param cipherdatalen [in] 密文数据长度
 * @param plaindata [in/out] 明文数据
 * @param plaindatalen [in/out] 明文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_decrypt_aes256_cbc(const unsigned char *key, unsigned int keylen,
                                    const unsigned char *iv, unsigned int ivlen,
                                    const unsigned char *cipherdata, unsigned int cipherdatalen,
                                    unsigned char *plaindata, unsigned int *plaindatalen);

/**
 * @brief 对称加密，使用AES算法（128位）ECB模式进行对称加密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param plaindata [in] 原数据
 * @param plaindatalen [in] 元数据长度
 * @param cipherdata [in/out] 密文数据
 * @param cipherdatalen [in/out] 密文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_encrypt_aes128_ecb(const unsigned char *key, unsigned int keylen,
                                    const unsigned char *iv, unsigned int ivlen,
                                    const unsigned char *plaindata, unsigned int plaindatalen,
                                    unsigned char *cipherdata, unsigned int *cipherdatalen);

/**
 * @brief 对称解密，使用AES算法（128位）ECB模式进行对称解密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param cipherdata [in] 密文数据
 * @param cipherdatalen [in] 密文数据长度
 * @param plaindata [in/out] 明文数据
 * @param plaindatalen [in/out] 明文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_decrypt_aes128_ecb(const unsigned char *key, unsigned int keylen,
                                    const unsigned char *iv, unsigned int ivlen,
                                    const unsigned char *cipherdata, unsigned int cipherdatalen,
                                    unsigned char *plaindata, unsigned int *plaindatalen);

/**
 * @brief 非对称加密，生成RSA算法的公私钥对
 * @param bit [in] 位数（1024/2048/4096等等）
 * @param pubkey [in/out] 公钥数据
 * @param pubkeylen [in/out] 公钥数据长度
 * @param privkey [in/out] 私钥数据
 * @param privkeylen [in/out] 私钥数据长度
 * @param password [in] 加密私钥数据的对称算法密钥，如果此实参为空，则私钥不进行加密；如果此参数不为空，将使用数据对私钥进行aes256-cbc加密了
 * @param passwordlen [in] 加密私钥数据的对称算法密钥长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_rsa_generatekeys(unsigned int bit,
                                unsigned char *pubkey, unsigned int *pubkeylen,
                                unsigned char *privkey, unsigned int *privkeylen,
                                unsigned char *password, unsigned int passwordlen);

/**
 * @brief 非对称加密，使用RSA算法进行非对称加密
 * @param pubkey [in] 公钥数据
 * @param pubkeylen [in] 公钥长度
 * @param plaindata [in] 原数据
 * @param plaindatalen [in] 原数据长度
 * @param cipherdata [in/out] 密文数据
 * @param cipherdatalen [in/out] 密文数据长度
 * @return 返回0表示成功，返回非零表示失败
 */
extern int kdk_rsa_encrypt(const unsigned char *pubkey, unsigned int pubkeylen,
                           const unsigned char *plaindata, unsigned int plaindatalen,
                           unsigned char *cipherdata, unsigned int *cipherdatalen);

/**
 * @brief 非对称加密，使用RSA算法进行解密
 * @param privkey [in] 私钥数据
 * @param privkeylen [in] 私钥数据长度
 * @param password [in] 加密私钥的密钥数据，如果此实参为空，则不对私钥进行解密操作
 * @param passwordlen [in] 加密私钥的密钥数据长度，如果此实参为0，与password为空相同处理
 * @param cipherdata [in] 密文数据
 * @param cipherdatalen [in] 密文数据长度
 * @param plaindata [in] 明文数据
 * @param plaindatalen [in] 明文数据长度
 * @return 返回0表示成功，返回非零表示失败
 */
extern int kdk_rsa_decrypt(const unsigned char *privkey, unsigned int privkeylen,
                           unsigned char *password, unsigned int passwordlen,
                           const unsigned char *cipherdata, unsigned int cipherdatalen,
                           unsigned char *plaindata, unsigned int *plaindatalen);
/**
 * @brief 数据签名，使用RSA算法进行数据签名
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param privkey [in] 私钥数据
 * @param privkeylen [in] 私钥数据长度
 * @param password [in] 加密私钥的密钥数据，如果此实参为空，则不对私钥进行解密操作
 * @param passwordlen [in] 加密私钥的密钥数据长度，如果此实参为0，与password为空相同处理
 * @param signdata [in/out] 签名数据
 * @param signdatalen [in/out] 签名数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_rsa_sign(const unsigned char *data, unsigned int datalen,
                        const unsigned char *privkey, unsigned int privkeylen,
                        const unsigned char *password, unsigned int passwordlen,
                        unsigned char *signdata, unsigned int *signdatalen);

/**
 * @brief 数据验签，使用RSA算法进行数据验签
 * @param signdata [in] 签名数据
 * @param signdatalen [in] 签名数据长度
 * @param pubkey [in] 公钥数据
 * @param pubkeylen [in] 公钥数据长度
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_rsa_verify(const unsigned char *signdata, unsigned int signdatalen,
                          const unsigned char *pubkey, unsigned int pubkeylen,
                          const unsigned char *data, unsigned int datalen);

/**
 * @brief MD5算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_md5(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);

/**
 * @brief SHA1算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_sha1(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);

/**
 * @brief SHA256算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_sha256(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);

/**
 * @brief SHA384算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_sha384(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);

/**
 * @brief SHA512算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0标识签名数据验签成功，返回非零表示失败
 */
extern int kdk_sha512(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);


/**
 * 国密算法专区
 */

/**
 * @brief 对称加密，使用SM4算法CBC模式进行对称加密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param data [in] 原数据
 * @param datalen [in] 元数据长度
 * @param encryptdata [in/out] 密文数据
 * @param encryptdatalen [in/out] 密文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_encrypt_sm4_cbc(const unsigned char *key, unsigned int keylen,
                                      const unsigned char *iv, unsigned int ivlen,
                                      const unsigned char *plaindata, unsigned int plaindatalen,
                                      unsigned char *cipherdata, unsigned int *cipherdatalen);

/**
 * @brief 对称解密，使用SM4算法CBC模式进行对称解密
 * @param key [in] 密钥数据
 * @param keylen [in] 密钥长度
 * @param iv [in] 向量
 * @param ivlen [in] 向量长度
 * @param cipherdata [in] 密文数据
 * @param cipherdatalen [in] 密文数据长度
 * @param plaindata [in/out] 明文数据
 * @param plaindatalen [in/out] 明文数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_cipher_decrypt_sm4_cbc(const unsigned char *key, unsigned int keylen,
                                         const unsigned char *iv, unsigned int ivlen,
                                         const unsigned char *cipherdata, unsigned int cipherdatalen,
                                         unsigned char *plaindata, unsigned int *plaindatalen);

/**
 * @brief 非对称加密，生成SM2算法的公私钥对
 * @param bit [in] 位数（256/512等等）
 * @param pubkey [in/out] 公钥数据
 * @param pubkeylen [in/out] 公钥数据长度
 * @param privkey [in/out] 私钥数据
 * @param privkeylen [in/out] 私钥数据长度
 * @param password [in] 加密私钥数据的对称算法密钥，如果此实参为空，则私钥不进行加密；如果此参数不为空，将使用数据对私钥进行aes256-cbc加密了
 * @param passwordlen [in] 加密私钥数据的对称算法密钥长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_sm2_generatekeys(unsigned int bit,
                                unsigned char *pubkey, unsigned int *pubkeylen,
                                unsigned char *privkey, unsigned int *privkeylen,
                                unsigned char *password, unsigned int passwordlen);

/**
 * @brief 非对称加密，使用SM2算法进行非对称加密
 * @param pubkey [in] 公钥数据
 * @param pubkeylen [in] 公钥长度
 * @param plaindata [in] 原数据
 * @param plaindatalen [in] 原数据长度
 * @param cipherdata [in/out] 密文数据
 * @param cipherdatalen [in/out] 密文数据长度
 * @return 返回0表示成功，返回非零表示失败
 */
extern int kdk_sm2_encrypt(const unsigned char *pubkey, unsigned int pubkeylen,
                           const unsigned char *plaindata, unsigned int plaindatalen,
                           unsigned char *cipherdata, unsigned int *cipherdatalen);

/**
 * @brief 非对称加密，使用SM2算法进行解密
 * @param privkey [in] 私钥数据
 * @param privkeylen [in] 私钥数据长度
 * @param password [in] 加密私钥的密钥数据，如果此实参为空，则不对私钥进行解密操作
 * @param passwordlen [in] 加密私钥的密钥数据长度，如果此实参为0，与password为空相同处理
 * @param cipherdata [in] 密文数据
 * @param cipherdatalen [in] 密文数据长度
 * @param plaindata [in] 明文数据
 * @param plaindatalen [in] 明文数据长度
 * @return 返回0表示成功，返回非零表示失败
 */
extern int kdk_sm2_decrypt(const unsigned char *privkey, unsigned int privkeylen,
                           unsigned char *password, unsigned int passwordlen,
                           const unsigned char *cipherdata, unsigned int cipherdatalen,
                           unsigned char *plaindata, unsigned int *plaindatalen);

/**
 * @brief 数据签名，使用SM2算法进行数据签名
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param privkey [in] 私钥数据
 * @param privkeylen [in] 私钥数据长度
 * @param password [in] 加密私钥的密钥数据，如果此实参为空，则不对私钥进行解密操作
 * @param passwordlen [in] 加密私钥的密钥数据长度，如果此实参为0，与password为空相同处理
 * @param signdata [in/out] 签名数据
 * @param signdatalen [in/out] 签名数据长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_sm2_sign(const unsigned char *data, unsigned int datalen,
                        const unsigned char *privkey, unsigned int privkeylen,
                        const unsigned char *password, unsigned int passwordlen,
                        unsigned char *signdata, unsigned int *signdatalen);

/**
 * @brief 数据验签，使用SM2算法进行数据验签
 * @param signdata [in] 签名数据
 * @param signdatalen [in] 签名数据长度
 * @param pubkey [in] 公钥数据
 * @param pubkeylen [in] 公钥数据长度
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @return int 返回0表示签名数据验签成功，返回非零表示失败
 */
extern int kdk_sm2_verify(const unsigned char *signdata, unsigned int signdatalen,
                          const unsigned char *pubkey, unsigned int pubkeylen,
                          const unsigned char *data, unsigned int datalen);

/**
 * @brief SM3算法计算HASH值
 * @param data [in] 原数据
 * @param datalen [in] 原数据长度
 * @param hash [in/out] HASH值
 * @param hashlen [in/out] HASH值长度
 * @return int 返回0表示签名数据验签成功，返回非零表示失败
 */
extern int kdk_sm3(const unsigned char *data, unsigned int datalen, unsigned char *hash, unsigned int *hashlen);

/**
 * @brief 随机数发生器
 * @param rnd  [in/out] 随机数内存
 * @param rndlen [in] 生成随机数的长度
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_rand(unsigned char *rnd, unsigned int rndlen);

/**
 * @brief BASE64编码
 * @param plaindata [in] 原数据
 * @param plaindatalen [in] 原数据长度
 * @param base64 [in/out] 编码后数据
 * @param base64len [in/out] 编码后数据长度
 * @return int 返回0表示编码成功，返回非零表示失败
 */
extern int kdk_base64_encode(const unsigned char *plaindata, unsigned int plaindatalen, unsigned char *base64, unsigned int *base64len);

/**
 * @brief BASE64解码
 * @param base64 [in] 编码数据
 * @param base64len [in] 编码数据长度
 * @param plaindata [in/out] 解码后数据
 * @param plaindatalen [in/out] 解码后数据长度
 * @return int 返回0表示编码成功，返回非零表示失败
 */
extern int kdk_base64_decode(const unsigned char *base64, unsigned int base64len, unsigned char *plaindata, unsigned int *plaindatalen);

#ifdef __cplusplus
}
#endif

#endif//LIBKYDATASEC_H
